# this one first, so we can find metacat, etc.
from env import env, token
from metacat.webapi import MetaCatClient, AuthenticationError, MCError

# other imports
import pytest
import time
import os
import sys

@pytest.fixture
def client(token):
    return MetaCatClient()

@pytest.fixture
def bearer_token():
    return get_bearer_token()

def get_bearer_token():
    token = os.environ.get("BEARER_TOKEN")
    if not token:
        token_file = os.environ.get("BEARER_TOKEN_FILE")
        if not token_file:
            uid = os.environ.get("ID", str(os.geteuid()))
            token_dir = os.environ.get("XDG_RUNTIME_DIR", "/tmp")
            token_file = token_dir + "/" + "bt_u" + uid
    token = open(token_file, "r").read().strip()
    return token

def test_token_auth(client, bearer_token):
    user, expiration = client.login_token(os.environ["USER"], bearer_token)
    assert(user == os.environ["USER"])
    assert(expiration > time.time())
    print(f"Authenticated as {user}")

def test_token_update(client):
    print("Starting test_token_update")
    sys.stdout.flush()

    initial_token = client.Token

    # wait a beat so token library file will have updated time...
    time.sleep(1)

    print("External auth update...")
    sys.stdout.flush()
    os.system("htgettoken -i hypot -a htvaultprod.fnal.gov ")
    os.system(f"metacat auth login -m token {os.environ['USER']}")  
    print("External auth update completed")
    sys.stdout.flush()

    client.token_update()

    assert(initial_token != client.Token)



"""
  This file has a commandline based integration tests for the metacat
  client and server for issue#33 about reparenting

  This suite needs to be run in-order, later tests depend on earlier ones. 
"""
import os
import time
import pytest
import json
import time

from env import env, token, auth, start_ds, tst_ds, tst_file_md_list

@pytest.fixture
def tst_ds_33(tst_ds):
    # a fresh dataset, our generic one with _33 on the end
    return tst_ds + "_33"

@pytest.fixture
def tst_file_md_list_33(tst_file_md_list):
    # a fresh metadata list with files with _33 in them and 
    # the fifth one being descended from the first two
    mdl = tst_file_md_list[:5].copy()
    for md in mdl:
        md["name"] = md["name"].replace(".txt","_33.txt")
    mdl[4]["parents"] = [{"namespace":x['namespace'], "name":x['name']} for x in mdl[:2]]
    yield mdl

#
# Setup for the real test ; these are clones of the tests in
# test_commandline with the alternate namespace,file metadata
#
def test_metacat_dataset_create_33(auth, tst_ds_33):
    with os.popen(f"metacat dataset create {tst_ds_33} ", "r") as fin:
        data = fin.read()
    assert data.find(tst_ds_33) > 0
    assert data.find("ataset") > 0
    assert data.find("eated") > 0  # dont fail if they fix spelling of cteated


def test_metacat_file_declare_many_33(auth, tst_ds_33, tst_file_md_list_33):
    with open("mdf", "w") as mdf:
        json.dump(tst_file_md_list_33[:4], mdf)
    with os.popen(f"metacat file declare-many mdf {tst_ds_33}", "r") as fin:
        data = fin.read()
    print(f"got: {data}")
    for md in tst_file_md_list_33[:4]:
        assert data.find(md["name"]) > 0
    with open("mdf", "w") as mdf:
        json.dump(tst_file_md_list_33[4:], mdf)
    with os.popen(f"metacat file declare-many mdf {tst_ds_33}", "r") as fin:
        data = fin.read()
    os.unlink("mdf")
    print(f"got: {data}")
    assert data.find(os.environ["USER"]) > 0
    for md in tst_file_md_list_33[5:]:
        assert data.find(md["name"]) > 0

    ns = tst_file_md_list_33[0]["namespace"]
    fn = tst_file_md_list_33[4]["name"]
    cmd = f"metacat file  show -jml {ns}:{fn}"
    print(f"running: {cmd}")
    with(os.popen(cmd , "r")) as fin:
        data=fin.read()
    print(f"got: {data}")
    
    # check response for requested parents
    assert data.find(tst_file_md_list_33[0]["name"]) > 0
    assert data.find(tst_file_md_list_33[1]["name"]) > 0
    # also check issue #60
    assert data.find('"created_timestamp"') > 0
    assert data.find('"created_timestamp": null') < 0


def test_update_33(auth, tst_ds_33, tst_file_md_list_33):
    ns = tst_file_md_list_33[0]["namespace"]
    n1 = tst_file_md_list_33[2]["name"]
    n2 = tst_file_md_list_33[3]["name"]
    fn = tst_file_md_list_33[4]["name"]
    cmd = f"metacat file update {ns}:{fn} --replace --parents {ns}:{n1},{ns}:{n2}"
    print(f"running: {cmd}")
    with(os.popen(cmd , "r")) as fin:
        data=fin.read()
    print(f"got: {data}")

    cmd = f"metacat file  show -jml {ns}:{fn}"
    print(f"running: {cmd}")
    with(os.popen(cmd , "r")) as fin:
        data=fin.read()
    print(f"got: {data}")
    
    # check response for new parents
    assert data.find(n1) > 0
    assert data.find(n2) > 0





"""
  Test for new feature --as-required on file creation
"""
import os
import time
import pytest
import json
import time

from env import env, token, auth, start_ds, tst_ds, tst_file_md_list


def test_as_required(auth, tst_ds, tst_file_md_list):
    tst_file_md_list = json.loads(json.dumps(tst_file_md_list))

    for md in tst_file_md_list:
        md["name"] = md["name"] + "_1"
        
    with open("mdf", "w") as mdf:
        json.dump(tst_file_md_list, mdf)

    with os.popen(f"metacat dataset create {tst_ds} ", "r") as fin:
        data = fin.read()

    print("first declare_dataset : got: ", data)

    with os.popen(f"metacat file declare-many mdf {tst_ds}", "r") as fin:
        data = fin.read()

    print("first declare: got: ", data)

    md = tst_file_md_list[0]
    os.popen(f"metacat file retire {md['namespace']}:{md['name']}")

    with os.popen(f"metacat file declare-many --as-required=unretire mdf {tst_ds}", "r") as fin:
        data = fin.read()

    print("second declare: got: ", data)
    os.unlink("mdf")

    assert data.find(os.environ["USER"]) > 0
    for md in tst_file_md_list[1:]:
        assert data.find(md["name"]) > 0
    




"""
  This file has a commandline based integration test for the metacat
  client and server.

  This suite needs to be run in-order, later tests depend on earlier ones. 
"""
import os
import time
import pytest
import json
import time

from env import env, token, auth, start_ds, tst_ds, tst_file_md_list

def test_metacat_dataset_create(auth, tst_ds):
    tst_ds =  tst_ds + "_cksum"
    with os.popen(f"metacat dataset create {tst_ds} ", "r") as fin:
        data = fin.read()
    assert data.find(tst_ds) > 0
    assert data.find("ataset") > 0
    assert data.find("eated") > 0  # dont fail if they fix spelling of cteated

def test_metacat_file_declare_bad_chksum_1(auth, tst_file_md_list, tst_ds):
    # add a file we did before, with ck1 on the filename, and
    # a 4-char adler32 checksum -- this should fail due to the checksum
    # being short.
    tst_ds =  tst_ds + "_cksum"
    md = tst_file_md_list[0].copy()
    md["name"] = "ck1_" + md["name"]
    md["checksums"]["adler32"] = md["checksums"]["adler32"][:4]
    print("metadata:\n", json.dumps(md))
    with open("mdf1", "w") as mdf:
        json.dump(md, mdf)
    with os.popen(f"metacat file declare -f mdf1 {tst_ds} 2>&1", "r") as fin:
        data = fin.read()
    print(f"got data: '{data}'")
    os.unlink("mdf1")
    assert data.find("validation error") > 0
    assert data.find("adler32: value is wrong length ") > 0

def test_metacat_file_declare_bad_chksum_2(auth, tst_file_md_list, tst_ds):
    # add a file we did before, with ck1 on the filename, and
    # a funky adler32 checksum, should report bad digits
    tst_ds =  tst_ds + "_cksum"
    md = tst_file_md_list[0].copy()
    md["name"] = "ck1_" + md["name"]
    md["checksums"]["adler32"] = "xy123456"
    print("metadata:\n", json.dumps(md))
    with open("mdf1", "w") as mdf:
        json.dump(md, mdf)
    with os.popen(f"metacat file declare -f mdf1 {tst_ds} 2>&1", "r") as fin:
        data = fin.read()
    print(f"got data: '{data}'")
    os.unlink("mdf1")
    assert data.find("validation error") > 0
    assert data.find("invalid digit") > 0




import os
import time
import pytest
import json

from env import env, token, auth


def test_issue_10_1(auth):
    # use case from issue 10
    start_ts = int(time.time())
    user = os.environ["USER"]
    os.system(f"""
        set -x
        metacat dataset create {user}:steve_test_retire_{start_ts}
        metacat file declare {user}:1mbtestfile.st2024{start_ts} {user}:steve_test_retire_{start_ts} -s 1024000 -c adler32:a0e10001
        metacat file declare {user}:1mbtestfile.st2024{start_ts}.child1 {user}:steve_test_retire_{start_ts} -s 1024000 -c adler32:a0e10001 --parents {user}:1mbtestfile.st2024{start_ts}
        metacat file declare {user}:1mbtestfile.st2024{start_ts}.child2 {user}:steve_test_retire_{start_ts} -s 1024000 -c adler32:a0e10001 --parents {user}:1mbtestfile.st2024{start_ts}
        metacat dataset files {user}:steve_test_retire_{start_ts}
        metacat file show {user}:1mbtestfile.st2024{start_ts} -l
        metacat file show {user}:1mbtestfile.st2024{start_ts}.child1 -l
        metacat file show {user}:1mbtestfile.st2024{start_ts}.child2 -l
    """)

    with os.popen(f"""
        set -x
        metacat query children '(files from {user}:steve_test_retire_{start_ts})'
    """) as fin:
         out1 = fin.read()

    os.system(f"""
        set -x
        metacat file retire {user}:1mbtestfile.st2024{start_ts}.child1
    """)

    with os.popen(f"""
        set -x
        metacat query children '(files from {user}:steve_test_retire_{start_ts})'
    """) as fin:
         out2 = fin.read()
    
    # Make sure our file is in the pre-remove query  but  not the post remove...
    print(f"comparing: '{out1}' '{out2}'")
    assert(out1.find( f"{user}:1mbtestfile.st2024{start_ts}.child1") >= 0)
    assert(out2.find( f"{user}:1mbtestfile.st2024{start_ts}.child1") < 0)

def test_issue_10_2(auth):
    # make sure we don't barf on updated/retired queries
    with os.popen(f"""
 metacat query 'files where updated_by == {os.environ['USER']} and updated_timestamp > now 
       and retired_by == {os.environ['USER']} and retired_timestamp > now and retired == false'
    """) as fin:
         out = fin.read()
    assert(out.strip() == "")



"""
  This file has a commandline based integration test for the metacat
  client and server.

  This suite needs to be run in-order, later tests depend on earlier ones. 
"""
import os
import time
import pytest
import json
import time

from env import env, token, auth, start_ds, tst_ds, tst_file_md_list


# need tests for at least:


def test_metacat_help_(auth):
    with os.popen("metacat help 2>&1 ", "r") as fin:
        data = fin.read()
    print(f"data: '{data}'")
    assert data.find("metacat") >= 0
    assert data.find("auth") > 0
    assert data.find("login") > 0
    assert data.find("version") > 0


def test_metacat_version_(auth):
    with os.popen("metacat version ", "r") as fin:
        data = fin.read()
        assert data.find("Server version") > 0
        assert data.find("Client version") > 0


# punting on non-token login types for now
# def test_metacat_auth_login_x509(auth, proxy):
#    with os.popen(f"metacat auth login -m x509 {os.environ['USER']}", "r") as fin:
#        data = fin.read()
#        assert(data.find(os.environ['USER']) > 0)
#        assert(data.find("User") >= 0)
#        assert(data.find("Expires") >= 0)
#
# def test_metacat_auth_login_services(auth, passwd):
#    with os.popen(f"metacat auth login -m token {os.environ['USER']}", "r") as fin:
#        data = fin.read()
#        assert(data.find(os.environ['USER']) > 0)
#        assert(data.find("User") >= 0)
#        assert(data.find("Expires") >= 0)


def test_metacat_auth_login_token(auth, token):
    with os.popen(f"metacat auth login -m token {os.environ['USER']}", "r") as fin:
        data = fin.read()
        assert data.find(os.environ["USER"]) > 0
        assert data.find("User") >= 0
        assert data.find("Expires") >= 0


def test_delay():
    time.sleep(20)


def test_metacat_auth_whoami(auth):
    with os.popen("metacat auth whoami", "r") as fin:
        data = fin.read()
        assert data.find(os.environ["USER"]) > 0
        assert data.find("User") >= 0
        assert data.find("Expires") >= 0


# Not bothering with proxy bits...
# def test_metacat_auth_mydn(auth):
#    with os.popen("metacat auth mydn", "r") as fin:
#        data = fin.read()
#        # check output


def test_metacat_auth_list(auth):
    with os.popen("metacat auth list", "r") as fin:
        data = fin.read()
        assert data.find("Token library") >= 0
        assert data.find(f'{os.environ["USER"]}/.token_library') >= 0
        assert data.find(os.environ["METACAT_SERVER_URL"]) >= 0


def test_metacat_auth_export(auth):
    with os.popen(
        f"metacat auth export {os.environ['METACAT_SERVER_URL']}", "r"
    ) as fin:
        data = fin.read()
        assert len(data) > 128


# punting on this for now...
# def test_metacat_auth_import(auth):
#    with os.popen("metacat auth import", "r") as fin:
#        data = fin.read()

# need to first create a namespace for the username
# also test that the second time it's created fails
def test_initial_namespace_create(auth):
    os.system(f'metacat namespace create {os.environ["USER"]}')
    with os.popen(f'metacat namespace create {os.environ["USER"]} 2>&1', "r") as fin:
        data = fin.read()
    assert data.find("exists") > 0

# jumping some file delcaration tests first, so we then have some files
# to make datasets of(?)
def test_metacat_dataset_create(auth, tst_ds):
    with os.popen(f"metacat dataset create {tst_ds} ", "r") as fin:
        data = fin.read()
    assert data.find(tst_ds) > 0
    assert data.find("ataset") > 0
    assert data.find("eated") > 0  # dont fail if they fix spelling of cteated


def test_metacat_file_declare(auth, tst_file_md_list, tst_ds):
    md = tst_file_md_list[0]
    with open("mdf1", "w") as mdf:
        json.dump(md, mdf)
    with os.popen(f"metacat file declare -f mdf1 {tst_ds} ", "r") as fin:
        data = fin.read()
    print(f"got data: '{data}'")
    os.unlink("mdf1")
    assert data.find(os.environ["USER"]) > 0
    assert data.find(md["name"]) > 0


def test_metacat_file_declare_many(auth, tst_ds, tst_file_md_list):
    with open("mdf", "w") as mdf:
        json.dump(tst_file_md_list[1:], mdf)
    with os.popen(f"metacat file declare-many mdf {tst_ds}", "r") as fin:
        data = fin.read()
    os.unlink("mdf")
    assert data.find(os.environ["USER"]) > 0
    for md in tst_file_md_list[1:]:
        assert data.find(md["name"]) > 0


def test_metacat_dataset_show(auth, tst_ds):
    with os.popen(f"metacat dataset show {tst_ds}", "r") as fin:
        data = fin.read()
    ns, dsname = tst_ds.split(":")
    assert data.find(ns) > 0
    assert data.find(dsname) > 0
    assert data.find(os.environ["USER"]) > 0


def test_metacat_dataset_files(auth, tst_ds, tst_file_md_list):
    with os.popen(f"metacat dataset files {tst_ds}", "r") as fin:
        data = fin.read()
    # should list all the files...
    for md in tst_file_md_list:
        assert data.find(md["name"]) > 0


def test_metacat_query_q_1(auth, tst_file_md_list, tst_ds):
    with open("qfl1", "w") as qf:
        qf.write(f"files from {tst_ds}")
    with os.popen("metacat query -q qfl1", "r") as fin:
        data = fin.read()
    os.unlink("qfl1")
    for md in tst_file_md_list[:-1]:
        assert data.find(md["name"]) >= 0

def test_metacat_query_q_ordered_1(auth, tst_file_md_list, tst_ds):
    with open("qfl1", "w") as qf:
        qf.write(f"files from {tst_ds} ordered")
    with os.popen("metacat query -q qfl1", "r") as fin:
        data = fin.read()
    os.unlink("qfl1")
    for md in tst_file_md_list[:-1]:
        assert data.find(md["name"]) >= 0

def test_metacat_query_q_2(auth, tst_file_md_list, tst_ds):
    with open("qfl1", "w") as qf:
        qf.write(f"files from {tst_ds}")
    with os.popen("metacat query -q qfl1", "r") as fin:
        data = fin.read()
    os.unlink("qfl1")
    for md in tst_file_md_list[:-1]:
        assert data.find(md["name"]) >= 0


def test_metacat_query_mql(auth, tst_file_md_list, tst_ds):
    with os.popen(f"metacat query files from {tst_ds}", "r") as fin:
        data = fin.read()
    for md in tst_file_md_list[:-1]:
        assert data.find(md["name"]) >= 0

def test_metacat_query_mql_count(auth, tst_file_md_list, tst_ds):
    with os.popen(f"metacat query --summary count files from {tst_ds}", "r") as fin:
        data = fin.read()
    lines = data.split("\n")
    assert int(lines[0][6:]) == len(tst_file_md_list)

def test_metacat_query_mql_keys(auth, tst_file_md_list, tst_ds):
    with os.popen(f"metacat query --summary keys files from {tst_ds}", "r") as fin:
        data = fin.read()
    md = tst_file_md_list[0]["metadata"]
    for k in md:
        assert(data.find(k) >= 0)

def test_metacat_query_mql_batch_size(auth, tst_file_md_list, tst_ds):
    with os.popen(f"metacat query --batch_size=2 files from {tst_ds}", "r") as fin:
        data = fin.read()
    for md in tst_file_md_list[:-1]:
        assert data.find(md["name"]) >= 0


def test_metacat_dataset_list(auth, tst_ds):
    with os.popen(f"metacat dataset list {os.environ['USER']}:*", "r") as fin:
        data = fin.read()
    assert data.find(tst_ds) >= 0


def test_metacat_dataset_add_subset(auth, tst_ds):
    tst_ds2 = tst_ds + "_super"
    os.system(f"metacat dataset create {tst_ds2}")
    with os.popen(f"metacat dataset add-subset {tst_ds2} {tst_ds} 2>&1", "r") as fin:
        data = fin.read()
    assert data == ""


def test_metacat_dataset_add_files(auth, tst_ds):
    query = (
        f'((files where creator="{os.environ["USER"]}") - (files from {tst_ds} limit 10)) limit 2'
    )
    with os.popen(f"metacat dataset add-files --query '{query}' {tst_ds} ", "r") as fin:
        data = fin.read()
    assert data.find("Added 2 files") >= 0


def test_metacat_dataset_update_fail(auth, tst_ds):
    md = '{"foo": "bar"}'
    with os.popen(f"metacat dataset update -j -m '{md}' {tst_ds} 2>&1", "r") as fin:
        data = fin.read()
        # check output
    assert data.find("Metadata parameter without a category") >= 0


def test_metacat_dataset_update(auth, tst_ds):
    md = '{"foo.baz": "bar"}'
    with os.popen(f"metacat dataset update -j -m '{md}' {tst_ds}", "r") as fin:
        data = fin.read()
        # check output
    assert data.find("foo.baz") >= 0
    assert data.find("bar") >= 0


def test_metacat_dataset_remove(auth, tst_ds):
    tst_ds2 = tst_ds + "_super"
    with os.popen(f"metacat dataset remove {tst_ds2}", "r") as fin:
        data = fin.read()
    assert data.strip() == ""
    with os.popen(f"metacat dataset remove {tst_ds2}", "r") as fin:
        data = fin.read()
    assert data.find("not found") >= 0


def test_metacat_namespace_create(auth, tst_ds):
    ns = tst_ds.replace(":", "_")
    with os.popen(f"metacat namespace create -j {ns}", "r") as fin:
        data = fin.read()
        # check output
    print("Got:", data)
    assert data.find('"name":') >= 0
    assert data.find(ns) >= 0


def x_test_metacat_namespace_list(auth):
    ns = tst_ds.replace(":", "_")
    with os.popen("metacat namespace list", "r") as fin:
        data = fin.read()
    assert data.find(ns) >= 0


def test_metacat_namespace_show(auth, tst_ds):
    ns = tst_ds.replace(":", "_")
    with os.popen(f"metacat namespace show {ns}", "r") as fin:
        data = fin.read()
    assert data.find(ns) >= 0


# Categories -- Don't know how to add them, they show up empty
#    in hypot, nothing to test?!?
#
# def x_test_metacat_category_list(auth):
#    with os.popen("metacat category list", "r") as fin:
#        data = fin.read()
#        # check output
#
# def x_test_metacat_category_show(auth):
#    with os.popen("metacat category show", "r") as fin:
#        data = fin.read()
#        # check output
#
def test_metacat_file_declare_sample(auth):
    with os.popen("metacat file declare-sample", "r") as fin:
        data = fin.read()
        # check output
    assert data.find('"namespace":') >= 0
    assert data.find('"name":') >= 0
    assert data.find('"size":') >= 0


# punting
def x_test_metacat_file_move(auth):
    with os.popen("metacat file move", "r") as fin:
        data = fin.read()
        # check output


# deprecated
def x_test_metacat_file_add(auth):
    with os.popen("metacat file add", "r") as fin:
        data = fin.read()
        # check output


def test_metacat_file_datasets(auth, tst_file_md_list, tst_ds):
    ns = tst_file_md_list[0]["namespace"]
    fname = tst_file_md_list[0]["name"]
    with os.popen(f"metacat file datasets {ns}:{fname}", "r") as fin:
        data = fin.read()
    assert data.find(tst_ds) >= 0


def test_metacat_file_update(auth, tst_file_md_list, tst_ds):
    ns = tst_file_md_list[0]["namespace"]
    fname = tst_file_md_list[0]["name"]
    md = '{"foo.bar": "baz"}'
    with os.popen(f"metacat file update -j -m '{md}' {ns}:{fname}", "r") as fin:
        data = fin.read()
    assert data.find("foo.bar") >= 0


def test_metacat_file_update_meta(auth, tst_file_md_list, tst_ds):
    ns = tst_file_md_list[1]["namespace"]
    fname = tst_file_md_list[1]["name"]
    md = '{"foo.bar": "baz"}'
    cmd=f"metacat file update-meta -f {ns}:{fname} '{md}'"
    print(cmd)
    with os.popen(cmd, "r") as fin:
        data = fin.read()
    print(data)
    cmd2 = f"metacat file show -j -m {ns}:{fname}"
    print(cmd2)
    with os.popen(cmd2 , "r") as fin:
        data2 = fin.read()
    print(data2)
    assert data2.find("foo.bar") >= 0


def test_metacat_dataset_remove_files(auth, tst_ds):
    query = f"(files from {tst_ds}) limit 10"
    with os.popen(
        f"metacat dataset remove-files --query '{query}' {tst_ds}", "r"
    ) as fin:
        data = fin.read()
    assert data.find("Removed 10 files") >= 0


def test_metacat_file_retire(auth, tst_file_md_list, tst_ds):
    fname = tst_file_md_list[4]["name"]
    ns = tst_file_md_list[4]["namespace"]
    with os.popen(f"metacat file retire {ns}:{fname}", "r") as fin:
        data = fin.read()
        # check output


def test_metacat_file_name_fid(auth, tst_file_md_list, tst_ds):
    # make sure file name and fid are consistent
    fname = tst_file_md_list[3]["name"]
    ns = tst_file_md_list[3]["namespace"]
    with os.popen(f"metacat file fid {ns}:{fname}", "r") as fin:
        data = fin.read()
        fid = data.strip()
    with os.popen(f"metacat file name --did {fid}", "r") as fin:
        data = fin.read()
        did = data.strip()
    assert f"{ns}:{fname}" == did


def test_metacat_file_show(auth, tst_file_md_list, tst_ds):
    # make sure file name and fid are consistent
    fname = tst_file_md_list[3]["name"]
    ns = tst_file_md_list[3]["namespace"]
    with os.popen(f"metacat file show -j -m {ns}:{fname}", "r") as fin:
        data = fin.read()
    # we should be able to load the json data
    md = json.loads(data)
    print("metadata:", data)
    assert md
    # and it should all be there...
    assert 'fid' in md
    assert md['name'] == fname
    assert md['namespace'] == ns

    assert md['size'] in (37 ,38, 39) 
    assert md['created_timestamp'] > 0
    assert md['creator'] == os.environ["USER"]
    assert 'updated_timestamp' in md
    assert 'updated_by' in md
    assert md['retired_by'] == None
    assert md['retired_timestamp'] == None
    assert 'adler32' in md['checksums']
    assert 'f.ds' in md['metadata']

def test_metacat_file_show2(auth, tst_file_md_list, tst_ds):
    # make sure file name and fid are consistent
    fname = tst_file_md_list[3]["name"]
    ns = tst_file_md_list[3]["namespace"]
    with os.popen(f"metacat file show -m {ns}:{fname}", "r") as fin:
        data = fin.read()
    assert data.find(fname) >= 0
    assert data.find(ns) >= 0


def test_metacat_validate_good(auth, tst_file_md_list, tst_ds):
    md = tst_file_md_list[0]["metadata"]
    with open("mdf1", "w") as mdf:
        json.dump(md, mdf)
    with os.popen(f"metacat validate mdf1", "r") as fin:
        data = fin.read()
    os.unlink("mdf1")
    assert data.strip() == ""


def test_metacat_validate_bad(auth, tst_file_md_list, tst_ds):
    md = tst_file_md_list[0]["metadata"]
    md["wrong"] = 1
    with open("mdf1", "w") as mdf:
        json.dump(md, mdf)
    with os.popen(f"metacat validate mdf1 2>&1", "r") as fin:
        data = fin.read()
    os.unlink("mdf1")
    assert data.find("wrong") >= 0


def test_metacat_named_query_create(auth, tst_ds):
    nqn = f"{os.environ['USER']}:tst_q_{start_ds}"
    with os.popen(
        f"metacat named_query create {nqn} 'files from {tst_ds}'", "r"
    ) as fin:
        data = fin.read()
        # check output


def test_metacat_named_query_show(auth, tst_ds):
    nqn = f"{os.environ['USER']}:tst_q_{start_ds}"
    with os.popen(f"metacat named_query show {nqn}", "r") as fin:
        data = fin.read()
    assert data.find(f"files from {tst_ds}") >= 0


def test_metacat_named_query_list(auth):
    nqn = f"{os.environ['USER']}:tst_q_{start_ds}"
    with os.popen("metacat named_query list", "r") as fin:
        data = fin.read()
    assert data.find(nqn) >= 0


def test_metacat_named_query_search(auth):
    nqn = f"{os.environ['USER']}:tst_q_{start_ds}"
    with os.popen(
        f"metacat named_query search 'queries matching {nqn[:-2]}*'", "r"
    ) as fin:
        data = fin.read()
    assert data.find(nqn) >= 0



"""
  This file has a commandline based integration test for the metacat
  client and server.

  This suite needs to be run in-order, later tests depend on earlier ones. 
"""
import os
import time
import pytest
import json
import time

from env import env, token, auth, start_ds, tst_ds, tst_file_md_list


# test having a dataset start with a number, add a file to it,
# and query it

def test_metacat_numeric_dataset_create(auth, tst_ds):
    tst_ds = tst_ds.replace(":",":2")
    with os.popen(f"metacat dataset create {tst_ds} ", "r") as fin:
        data = fin.read()
    assert data.find(tst_ds) > 0
    assert data.find("ataset") > 0
    assert data.find("eated") > 0  # dont fail if they fix spelling of cteated

def test_metacat_numeric_ds_declare(auth, tst_file_md_list, tst_ds):
    tst_ds = tst_ds.replace(":",":2")
    md = tst_file_md_list[0].copy()
    md["name"] = md["name"] + "x"
    with open("mdfn", "w") as mdf:
        json.dump(md, mdf)
    with os.popen(f"metacat file declare -f mdfn {tst_ds} ", "r") as fin:
        data = fin.read()
    print(f"got data: '{data}'")
    os.unlink("mdfn")
    assert data.find(os.environ["USER"]) > 0
    assert data.find(md["name"]) > 0

def test_metacat_numeric_query_q(auth, tst_file_md_list, tst_ds):
    tst_ds = tst_ds.replace(":",":2")
    md = tst_file_md_list[0].copy()
    md["name"] = md["name"] + "x"
    with open("qfl1", "w") as qf:
        qf.write(f"files from {tst_ds}")
    with os.popen("metacat query -q qfl1", "r") as fin:
        data = fin.read()
    os.unlink("qfl1")
    assert data.find(md["name"]) >= 0



"""
  This file has a commandline based integration tests for the metacat
  client and server for issue#33 about reparenting

  This suite needs to be run in-order, later tests depend on earlier ones. 
"""
import os
import time
import pytest
import json
import time

from env import env,token,auth, tst_ds

def test_metacat_query_explain_57(auth, tst_ds):
    with os.popen(f"metacat query --explain files from {tst_ds}", "r") as fin:
        data = fin.read()
    print(f"got: {data}")
    assert data.find(tst_ds) > 0
    assert data.find("select") >= 0



from env import env

from metacat.mql.meta_evaluator import MetaEvaluator
from metacat.mql.mql10 import _Parser, MQLQuery

# routine to make short tests: evaluate an expression against some metadata

def eval_expr(e, md):
    """ evaluate an expression for the given metadata ... """
    class mock_file:
        def __init__(self, md):
            self.md = md
        def metadata(self):
            return self.md
    f = mock_file(md)      # fake file with this metadata
    me = MetaEvaluator()   # metadata expression evaluator

    # we parse a whole "files where ..." expression, then pick out the where clause to evaluate

    pe = MQLQuery.parse(f"files where {e}")
    return me( f, pe.Tree.D['query'].Wheres )


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# testing == and != 

def test_exp_1():
    assert( eval_expr("c.n1 == 10", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_2():
    assert( not eval_expr("c.n1 != 10", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_3():
    assert( eval_expr("c.n2 == 20", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_4():
    assert( not eval_expr("c.n2 != 20", {"c.n1": 10, "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# testing various "in" combinations

def test_exp_5():
    assert( eval_expr("c.n1 in (9,10,11)", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_6():
    assert( eval_expr("c.n1 in 9:11", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_5n():
    assert( not eval_expr("c.n1 not in (9,10,11)", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_6n():
    assert( not eval_expr("c.n1 not in 9:11", {"c.n1": 10, "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# test 'present'

def test_exp_7():
    assert( eval_expr("c.n1 present", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_8():
    assert( not eval_expr("c.n1 not present", {"c.n1": 10, "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# test constant in metadata-field

def test_exp_9():
    assert( eval_expr("10 in c.n1", {"c.n1": [8,9,10], "c.n2": 20} ) )

def test_exp_10():
    assert( not eval_expr("6 in c.n1", {"c.n1": [8,9,10], "c.n2": 20} ) )

def test_exp_10_1():
    assert( eval_expr("6 not in c.n1", {"c.n1": [8,9,10], "c.n2": 20} ) )

def test_exp_10_2():
    assert( not eval_expr("6 in c.n1", {"c.n1": [8,9,10], "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# test less/greater/equal

def test_exp_11():
    assert( eval_expr("c.n1 <= 10", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_12():
    assert( eval_expr("c.n1 >= 10", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_13():
    assert( eval_expr("c.n1 < 11", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_14():
    assert( eval_expr("c.n1 > 9", {"c.n1": 10, "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# test regex match

def test_exp_15():
    assert( eval_expr("c.s1 ~ 'a.*b'", {"c.s1": "xaxyzby", "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# test and, or...

def test_exp_20():
    assert( eval_expr("(c.n1 < 11) and (c.n2 < 21)", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_21():
    assert( eval_expr("(c.n1 < 11) or (c.n2 < 10)", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_22():
    assert( eval_expr("(c.n1 < 9) or (c.n2 < 21)", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_23():
    assert( eval_expr("(c.n1 in 8:11) or (c.n2 < 19)", {"c.n1": 10, "c.n2": 20} ) )

def test_exp_24():
    assert( eval_expr("(c.n1 in (8,9,10)) or (c.n2 < 19)", {"c.n1": 10, "c.n2": 20} ) )

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# bug cases from Steven Timm

def test_exp_30():
    assert( eval_expr(
        "core.runs[1]>=26785 and core.run_type=hd-protodune and core.data_tier=raw",
        {"core.runs": [1, 26785], "core.run_type": "hd-protodune", "core.data_tier": "raw"},
    ))

def test_exp_31():
    assert( eval_expr(
        "core.runs[1] in (26785,26789,26790,26791,26792,26793,26794) and core.run_type=hd-protodune and core.data_tier=raw",
        {"core.runs": [1, 26785], "core.run_type": "hd-protodune", "core.data_tier": "raw"},
    ))

def test_exp_32():
    assert( eval_expr(
        "core.runs[1] in 26785:26794  and core.run_type=hd-protodune and core.data_tier=raw",
        {"core.runs": [1, 26785], "core.run_type": "hd-protodune", "core.data_tier": "raw"},
    ))




FileAttributes = [      # file attributes which can be used in queries
            "creator", "created_timestamp", "name", "namespace", "size",
            "retired", "retired_by", "retired_timestamp", "updated_by",  
            "updated_timestamp"
]  

DatasetAttributes = [
    "creator", "created_timestamp", "namespace", "name",
    "description", "frozen", "monotonic"
]
